#!/usr/bin/perl
use File::ExtAttr ':all';
use IO::File;
use Getopt::Long;
use File::Find;

my $VERSION="0.2";

my $info = <<INFO;
Get extended attributes for HFS+ v$VERSION
.....................................
(c) 2008 fG! - http://reverse.put.as - reverse\@put.as

Usage: $0 --object <path or file> --command <command> --attribute <attribute>

Options:
--object <path or file>\t = Directory or file to read extended attributes from
--command <command>\t = available commands: read, list, listcontent (default), listall, write
where:
  read: read specific attribute
  list: list attributes available for the object
  listcontent: list attributes and content for the object
  listall: find all files/dirs with extended attributes and content
  write: create attribute with specified content
--attribute <attribute>\t = Specific attribute name we want to read or write
--content <content>\t = content to write to the attribute
--help\t\t\t = show help
--debug\t\t\t = print debug information
INFO

my $header = <<HEADER;
Get extended attributes for HFS+ v$VERSION
.....................................
(c) 2008 fG! - http://reverse.put.as - reverse@put.as

HEADER

sub help
{
	print $info;
	exit 0;
}

sub errormsg
{
	warn "-------------------------------\nERROR: $_[0]-------------------------------\n\n";
}

sub loadFiles
{
	find(\&mySub,"$_[0]"); #custom subroutine find, parse $dir
}

# following gets called recursively for each file in $dir, check $_ to see if you want the file!
sub mySub()
{
	push @files, $File::Find::name ; 
}

# default options
my $debug=0;
my $help = '';
my $object = '';
my $content = '';
my $attribute = '';
my $command = 'listcontent';

my ($fh, $result);

# define options
GetOptions('debug' => \$debug, 'help' => \$help, 'object=s' => \$object, 'attribute=s' => \$attribute, 'command=s' => \$command, 'content=s' => \$content);

# process command options
if ($help == 1) { &help; }
# object is always required!
unless ($object)
{
	errormsg("Missing required options\nObject is required!\n");
	&help;
}
#check if object exists
if (! -e $object)
{
	errormsg("Object not found\nVerify if path or file exists!\n");
	exit(1);
}

if (lc($command) eq "write" && !$content)
{
	errormsg("Content to be written is empty!\n");
	exit(1);
}

# and start the fun...
print $header;
# read a specific attribute from an object (requires the attribute)
if (lc($command) eq "read")
{
	unless($attribute)
	{
		 errormsg("Missing required options\nread command requires an attribute\n");
		 &help;
	}
	print "DEBUG: Creating filehandle...\n" if $debug;
	$fh = new IO::File($object);
	print "Reading attribute $attribute from $object...\n";
	$result = getfattr($fh,$attribute);
	printf("Content is:\n%s\n", $result);
	$fh->close;
	print "Finished!\n";
}
# list available attributes for an object
elsif (lc($command) eq "list")
{
	print "DEBUG: Creating filehandle...\n" if $debug;
	$fh = new IO::File($object);
	print "Listing all attributes from $object :\n";
	foreach (listfattr($fh))
	{
		print "Attribute name: $_\n";
	}
	$fh->close;
	print "Finished!\n";
}
# list available attributes and their content for an object
elsif (lc($command) eq "listcontent")
{
	print "DEBUG: Creating filehandle...\n" if $debug;
	$fh = new IO::File($object);
	print "Listing all attributes and content from $object :\n";
	foreach (listfattr($fh))
	{	
		$attrcontent = getfattr($fh,$_);
		print "Attribute name: $_ with content: $attrcontent\n";
	}
	$fh->close;
	print "Finished!\n";
}
# find all files/dir in the given path and list their attributes/content
elsif (lc($command) eq "listall")
{
	my @files = ();
	# add the directory itself to the list
	push(@files,$object);
    print "Creating list of files...\n";
	loadFiles($object);
	foreach (@files)
	{
		print "Finding and listing all attributes and content from $_ :\n";
		$fh = new IO::File($_);
		foreach $tempattrib (listfattr($fh))
		{
			$attrcontent = getfattr($fh,$tempattrib);
			print "File/Dir name: $_ Attribute name: $tempattrib with content: $attrcontent\n";
		}
	}
	$fh->close;
	print "Finished!\n";
}
# write attribute to specified file/dir
elsif (lc($command) eq "write")
{
	$fh = new IO::File($object);
	print "Creating attribute with specified content...\n";
	setfattr($fh,$attribute,$content) || errormsg("Can't write attribute!\n");
	print "Verifying write...\n";
	$attrcontent = getfattr($fh,$attribute);
	print "Attribute written was \"$attribute\" with content \"$content\"\n";
	$fh->close;
	print "Finished!\n";	
}
